import { Machine, State, On, Do, Goto, Spawn, Unspawn, interpret, Interpreter_T, send, Event_T } from '../index';

const wait = (ms:number)=>new Promise((resolve)=>{ setTimeout(()=>{ resolve(1); }, ms); });
const makeRequest = (ctx,e,self)=>{ send(ctx.serverActor, ['received-request',self]); };
const sendResponse = (ctx,e,self)=>{ send(ctx.clientActor, ['received-response',self]); };
const startTimer = async (ctx,e,self)=>{ await wait(500); send(self, ['timer-finished',null]); }
const log = (ctx, e, self:Interpreter_T)=>{ console.log(self.state); };

type Sc = 
  | 'idle' 
  | 'making-request' 
  | 'awaiting-response';
type Ec = 
  | ['entry',null] 
  | ['received-response',Interpreter_T]
  | ['server-created', Interpreter_T];
type Cc = 
  | {serverActor?:Interpreter_T};

const client = 
  Machine(
    State('idle',
      On('entry',
        Do(log),
      ),
      On('server-created',
        
        Do((_ctx,[_eventName,serverActor],self)=>{ self.context.serverActor=serverActor; }),
        Goto('making-request')
      )
    ),
    State('making-request',
      On('entry', 
        Do(log),
        Do(makeRequest),
        Goto('awaiting-response')
      ),
    ),
    State('awaiting-response',
      On('entry',
        Do(log),
      ),
      On('received-response', 
        Do(log),
        Goto('making-request')
      ),
    ),
  );

type Ss = 
  | 'awaiting-request' 
  | 'sending-response';
type Es = 
  | ['entry',null] 
  | ['received-request',Interpreter_T]
  | ['timer-finished', null];
type Cs = {clientActor?:Interpreter_T};

const server = 
  Machine(
    State('awaiting-request',
      On('entry',
        Do(log),
      ),
      On('received-request', 
        Do((_ctx,[_eventName,clientActor],self)=>{ self.context.clientActor=clientActor; }),
        Goto('sending-response')
      ),
    ),
    State('sending-response',
      On('entry', 
        Do(log),
        Do(startTimer)
      ),
      On('timer-finished',
        Do(sendResponse),
        Goto('awaiting-request')
      )
    ),
  );

const clientActor = interpret(client, {context:{}});
const serverActor = interpret(server, {context:{}});
send(clientActor, ['server-created', serverActor]);